@@ -4,7 +4,8 @@ import plotly.graph_objects as go
 from plotly.subplots import make_subplots
 from textblob import TextBlob
 from insightsearch.sentiment_analyze import *
-
+from tqdm import tqdm
+tqdm.pandas()
 
 
 
@@ -16,7 +17,6 @@ class Aspectanalyze(Sentimentanalyze):
         self.column_name = column_name
         self.lines = [l for l in self.df["reviewText"]]
 
-
     def _aspect_list(self, line, candidate_aspects, aspect_opinion_dict):
         blob = TextBlob(line)
 
@@ -25,11 +25,12 @@ class Aspectanalyze(Sentimentanalyze):
                 first_w_index = blob.words.index(a.split()[0])
 
                 last_w_index = blob.words.index(a.split()[-1])
+
                 # chechking whether word_index-1 is adj if yes will add to list
                 pref_words = []
-                if self.index_valid(first_w_index - 1, blob.tags):
+                if self.index_valid(first_w_index - 1 , blob.tags):
                     if blob.tags[first_w_index - 1][1] in [
-                        "RB",
+                       "RB",
                         "RBR",
                         "RBS",
                         "JJ",
@@ -39,9 +40,22 @@ class Aspectanalyze(Sentimentanalyze):
                         "very",
                         "really","much"
                     ]:
-                        pref_words.append(
-                            blob.tags[first_w_index - 1][0].lower().strip()
-                        )
+
+                        if self.index_valid(first_w_index - 2, blob.tags) and blob.tags[first_w_index - 2][0].lower().strip() in [
+                               "not"] :
+
+                            pref_words.append(
+                               "not" + str(blob.tags[first_w_index - 1][0].lower().strip())
+                            )
+
+                        else:
+                            pref_words.append(
+                               blob.tags[first_w_index - 1][0].lower().strip()
+                            )
+
+
+
+
 
                 if len(pref_words) > 0:
                     if self.index_valid(first_w_index - 2, blob.tags):
@@ -54,10 +68,50 @@ class Aspectanalyze(Sentimentanalyze):
                             "JJS",
                         ] and blob.tags[first_w_index - 2][0].lower().strip() not in [
                             "very",
-                            "really","much"
+                            "really","much","and"
+                        ]:
+                            if self.index_valid(first_w_index - 3, blob.tags) and blob.tags[first_w_index - 3][0].lower().strip() in [
+                               "not"] :
+
+                                pref_words.append(
+                               "not" + str(blob.tags[first_w_index - 2][0].lower().strip())
+                            )
+
+                            else:
+                                pref_words.append(
+                               blob.tags[first_w_index - 2][0].lower().strip()
+
+                            )
+                    if self.index_valid(first_w_index - 2, blob.tags):
+                        if blob.tags[first_w_index - 2][1] in [
+                            "RB",
+                            "RBR",
+                            "RBS",
+                            "JJ",
+                            "JJR",
+                            "JJS",
+                        ] and blob.tags[first_w_index - 2][0].lower().strip() in [
+                            "and"
                         ]:
                             pref_words.append(
-                                blob.tags[first_w_index - 2][0].lower().strip()
+                                blob.tags[first_w_index - 3][0].lower().strip()
+
+                            )
+
+                    if self.index_valid(first_w_index - 3, blob.tags):
+                        if blob.tags[first_w_index - 3][1] in [
+                            "RB",
+                            "RBR",
+                            "RBS",
+                            "JJ",
+                            "JJR",
+                            "JJS",
+                        ] and blob.tags[first_w_index - 3][0].lower().strip() not in [
+                            "very",
+                            "really","much","and"
+                        ]:
+                            pref_words.append(
+                                blob.tags[first_w_index - 3][0].lower().strip()
                             )
 
                 attr = "".join(pref_words)
@@ -71,9 +125,7 @@ class Aspectanalyze(Sentimentanalyze):
 
                 after_words = []
 
-                if self.index_valid(first_w_index + 1, blob.tags) and blob.words[
-                    last_w_index + 1
-                ].lower() in ["am", "is", "are", "was", "were"]:
+                if self.index_valid(first_w_index + 1, blob.tags) and blob.words[first_w_index + 1].lower() in ["am", "is", "are", "was", "were"]:
                     if self.index_valid(first_w_index + 2, blob.tags):
 
                         if blob.tags[last_w_index + 2][1] in [
@@ -85,13 +137,14 @@ class Aspectanalyze(Sentimentanalyze):
                             "JJS",
                         ] and blob.tags[last_w_index + 2][0].lower().strip() not in [
                             "very",
-                            "really",
-                        ]:
+                            "really","always"
+                        ] :
                             after_words.append(
                                 blob.tags[last_w_index + 2][0].lower().strip()
                             )
-                if len(after_words) > 0:
+
                     if self.index_valid(first_w_index + 3, blob.tags):
+
                         if blob.tags[last_w_index + 3][1] in [
                             "RB",
                             "RBR",
@@ -99,51 +152,83 @@ class Aspectanalyze(Sentimentanalyze):
                             "JJ",
                             "JJR",
                             "JJS",
-                        ] and blob.tags[first_w_index - 1][0].lower().strip() not in [
-                            "very",
-                            "really",
-                        ]:
-                            after_words.append(
-                                blob.tags[last_w_index + 3][0].lower().strip()
+                        ] and blob.tags[last_w_index + 3][0].lower().strip() in [
+                            "and"
+
+                        ] :
+                            if self.index_valid(first_w_index + 4, blob.tags):
+
+                                after_words.append(
+                                    blob.tags[last_w_index + 4][0].lower().strip()
                             )
 
-                if self.index_valid(first_w_index + 2, blob.tags):
-                    if blob.tags[last_w_index + 2][0].lower() in ["very", "really"]:
 
+                    if len(after_words) > 0:
                         if self.index_valid(first_w_index + 3, blob.tags):
-                            after_words.append(
-                                blob.tags[last_w_index + 3][0].lower().strip()
+                            if blob.tags[last_w_index + 3][1] in [
+                                "RB",
+                                "RBR",
+                                "RBS",
+                                "JJ",
+                                "JJR",
+                                "JJS",
+                            ] and blob.tags[first_w_index + 3][0].lower().strip() not in [
+                                "very",
+                                "really","much","and"
+                            ]:
+                                after_words.append(
+                                    blob.tags[last_w_index + 3][0].lower().strip()
+                                )
+                    if self.index_valid(first_w_index + 4, blob.tags):
+                        if blob.tags[last_w_index + 4][1] in [
+                            "RB",
+                            "RBR",
+                            "RBS",
+                            "JJ",
+                            "JJR",
+                            "JJS",
+                        ] and blob.tags[first_w_index + 3][0].lower().strip() in [
+                            "and"
+                        ]:
+                            if self.index_valid(first_w_index + 4, blob.tags):
+
+                                after_words.append(
+                                        blob.tags[last_w_index + 4][0].lower().strip()
                             )
-                attr = " ".join(after_words)
-                if len(after_words) == 2 and after_words[0] in ["not", "n't"]:
-                    attr = "".join(after_words)
 
-                if str(a.strip()) in list(aspect_opinion_dict.keys()):
 
-                    aspect_opinion_dict[a] = " ".join(
-                        [aspect_opinion_dict[a] + " " + attr]
-                    ).strip()
-                else:
-                    aspect_opinion_dict[str(a).strip()] = attr
+
+                    if self.index_valid(first_w_index + 2, blob.tags):
+                        if blob.tags[last_w_index + 2][0].lower() in ["very", "really"]:
+
+                            if self.index_valid(first_w_index + 3, blob.tags):
+                                after_words.append(
+                                    blob.tags[last_w_index + 3][0].lower().strip()
+                                )
+                    attr = " ".join(after_words)
+
+                    if str(a.strip()) in list(aspect_opinion_dict.keys()):
+
+                        aspect_opinion_dict[a] = " ".join(
+                            [aspect_opinion_dict[a] + " " + attr]
+                        ).strip()
+                    else:
+                        aspect_opinion_dict[str(a).strip()] = attr
+            else:
+                pass
 
     def aspect_generator(self):
 
         self.candidate_aspects_dict = {}
         self.aspect_opinion_dict = {}
         aspects = []
+        no_reviews=len(self.lines)
+        for line in tqdm(self.lines,desc="Collecting Aspects And Opinions From {} Reviews".format(no_reviews)):
 
-        for line in self.lines:
-            candidate_aspects = []
             blob = TextBlob(line)
 
-            nouns_1 = [
-                aspects.append(n)
-                for n, t in blob.tags
-                if t in ["NN", "NNS", "NNP", "NNPS"]
-            ]
-            candidate_aspects = [
-                n for n, t in blob.tags if t in ["NN", "NNS", "NNP", "NNPS"]
-            ]
+            nouns_1 = [aspects.append(n) for n, t in blob.tags if t in ["NN", "NNS", "NNP", "NNPS"] ]
+            candidate_aspects = [n for n, t in blob.tags if t in ["NN", "NNS", "NNP", "NNPS"] ]
 
             self._aspect_list(line, candidate_aspects, self.aspect_opinion_dict)
         keys = list(self.aspect_opinion_dict.keys())
@@ -151,6 +236,7 @@ class Aspectanalyze(Sentimentanalyze):
 
             if a in self.stop_words:
                 self.aspect_opinion_dict.pop(a)
+
         final_df = pd.DataFrame(
             self.aspect_opinion_dict.items(), columns=["aspects", "opinions"]
         )
@@ -174,7 +260,6 @@ class Aspectanalyze(Sentimentanalyze):
             aspect_opinion_dict = self.aspect_opinion_dict
 
         except AttributeError:
-            print("Initiating the function aspect_generator")
             self.aspect_generator()
         aspect_name = list(self.rslt_df["aspects"][:9])
         aspect_opinion = list(self.rslt_df["opinion_list"][:9])
@@ -189,9 +274,19 @@ class Aspectanalyze(Sentimentanalyze):
         rows = [1, 1, 1, 2, 2, 2, 3, 3, 3]
 
         for s in range(9):
-            op = [w[0] for w in aspect_opinion[s][:5]][::-1]
+            try:
+                op = [w[0] for w in aspect_opinion[s][:5]][::-1]
+            except:
+                if len(aspect_opinion[s]) > 0:
 
-            num = [w[1] for w in aspect_opinion[s][:5]][::-1]
+                        op=[w[0] for w in aspect_opinion[s][:len(aspect_opinion[s])]][::-1]
+            try:
+
+                num = [w[1] for w in aspect_opinion[s][:5]][::-1]
+            except:
+
+               if len(aspect_opinion[s]) > 0:
+                  num=[w[1] for w in aspect_opinion[s][:len(aspect_opinion[s])]][::-1]
 
             fig = fig.add_trace(
                 go.Bar(
